#include <qaction.h>
#include <qlayout.h>
#include <qobjectlist.h>
#include <qpushbutton.h>
#include <qstring.h>
#include <qtable.h>

#include "actionsdialog.h"

ActionsDialog::ActionsDialog(QObjectList *actions, QWidget *parent)
    : QDialog(parent)
{
    actionsTable = new QTable(actions->count(), 2, this);
    actionsTable->horizontalHeader()->setLabel(0, tr("Description"));
    actionsTable->horizontalHeader()->setLabel(1, tr("Shortcut"));
    actionsTable->verticalHeader()->hide();
    actionsTable->setLeftMargin(0);
    actionsTable->setColumnReadOnly(0, true);

    QAction *action = static_cast<QAction*>(actions->first());
    int row = 0;
    
    while (action) {
        actionsTable->setText(row, 0, action->text());
        actionsTable->setText(row, 1, QString(action->accel()));

        actionsList.append(action);
        action = static_cast<QAction*>(actions->next());
        ++row;
    }

    QPushButton *okButton = new QPushButton(tr("&OK"), this);
    QPushButton *cancelButton = new QPushButton(tr("&Cancel"), this);

    connect(actionsTable, SIGNAL(currentChanged(int, int)),
            this, SLOT(recordAction(int, int)));
    connect(actionsTable, SIGNAL(valueChanged(int, int)),
            this, SLOT(validateAction(int, int)));
    connect(okButton, SIGNAL(clicked()), this, SLOT(accept()));
    connect(cancelButton, SIGNAL(clicked()), this, SLOT(reject()));

    QHBoxLayout *buttonLayout = new QHBoxLayout;
    buttonLayout->setSpacing(8);
    buttonLayout->addStretch(1);
    buttonLayout->addWidget(okButton);
    buttonLayout->addWidget(cancelButton);

    QVBoxLayout *mainLayout = new QVBoxLayout(this);
    mainLayout->setMargin(8);
    mainLayout->setSpacing(8);
    mainLayout->addWidget(actionsTable);
    mainLayout->addLayout(buttonLayout);

    setCaption(tr("Edit Actions"));
}

void ActionsDialog::accept()
{
    for (int row = 0; row < (int)actionsList.size(); ++row) {
        QAction *action = actionsList[row];
        action->setAccel(QKeySequence(actionsTable->text(row, 1)));
    }

    QDialog::accept();
}

void ActionsDialog::recordAction(int row, int column)
{
    oldAccelText = actionsTable->item(row, column)->text();
}

void ActionsDialog::validateAction(int row, int column)
{
    QTableItem *item = actionsTable->item(row, column);
    QString accelText = QString(QKeySequence(item->text()));

    if (accelText.isEmpty() && !item->text().isEmpty())
        item->setText(oldAccelText);
    else
        item->setText(accelText);
}
